import datetime
import random


class SecurityEvent:

    def __init__(self, event, testing: bool):
        self.detail: str = event["detail"]
        self.type: str = event["type"]
        self.probability: float = None
        self.attackerIP = self.get_attacker_ip_or_none(event)
        self.subType: str = self.get_subtype_or_none(event)
        self.victimIP = self.get_victimIP_or_none(event)
        self.victimPort = self.get_victimPort_or_none(event)
        self.detectionTime = self.get_detectionTime_or_none(event, testing)
        self.reporterId = self.get_reporter_ip(event) if not testing else random.choice(["192.168.1.1", "192.168.1.2", "192.168.1.3"])

    def get_attacker_ip_or_none(self, event):
        try:
            return event["source"]["ip"]
        except:
            return None

    def get_subtype_or_none(self, event):
        try:
            return event["subtype"]
        except:
            return None

    def get_victimIP_or_none(self, event):
        try:
            return event["targets"][0]["ip"]
        except:
            return None

    def get_victimPort_or_none(self, event):
        try:
            return event["attributes"]["ports"][0]
        except:
            return None

    def get_detectionTime_or_none(self, event, testing: bool):
        try:
            result = event["time"]
        except:
            result = None

        if testing:
            result = self.get_date_mod_24_hours(result).strftime("%Y-%m-%d %H:%M:%S")
        return result

    def get_date_mod_24_hours(self, date_str: str) -> datetime.datetime:
        date = datetime.datetime.fromisoformat(date_str)
        time_diff = datetime.datetime.now() - date
        return date + datetime.timedelta(days=time_diff.days)

    def get_reporter_ip(self, event):
        pass
